screen = {
		width = love.graphics.getWidth(),
		height = love.graphics.getHeight()
	}

--Loading Files
require "console"
require "planet"
--require "stars"

--Variables
local line = {
	pos = {
		{0, 0},
		{0, 0}
	},
	color = {255, 0, 0, 126}
}

simulation = true
simulation_speed = 1

function love.load()
	console:init()
	planet:newWorld(0, 0, true, 64)
	--stars:init()
end

function love.update(dt)
	dt = dt * simulation_speed 
	local mx, my = love.mouse.getPosition()

	if love.mouse.isDown("l") then
		line.pos[2][1] = mx
		line.pos[2][2] = my
	end

	planet:update(dt)
	console:update(dt)
end

function love.draw()
	--stars:draw()
	console:draw()
	planet:draw()
	planet:draw_physics_debug()

	if love.mouse.isDown("l") then
		love.graphics.setColor(line.color)
		love.graphics.setLineWidth(1)
		love.graphics.line(line.pos[1][1], line.pos[1][2], line.pos[2][1], line.pos[2][2])
	end
	--Debug
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.print("fps: "..love.timer.getFPS()..
						"\nTotal Planets: "..planet:countPlanets()..
						"\n    Static: "..planet:countStaticPlanets()..
						"\n    Dynamic: "..planet:countDynamicPlanets()..
						"\nSimualtion Running: "..tostring(simulation)..
						"\nSimulation Speed: "..simulation_speed, 12, 12)
end

function love.textinput(t)
	console:textinput(t)
end

function love.keypressed(key)
	if key == "escape" then love.event.push("quit") end
	if key == "tab" then console:toggle() end

	if not console:isVisible() then
		if key == " " then simulation = not simulation end
		if key == "w" then
			for i=1, 64 do
				planet:new(math.random() * screen.width, math.random() * screen.height, 6, false, 3, 0.5)
			end
		end
	end
	console:keypressed(key)
end

function love.mousepressed(mouse_x, mouse_y, key)
	if key == "l" then
		line.pos[1][1] = mouse_x
		line.pos[1][2] = mouse_y
	elseif key == "r" then
		planet:new(mouse_x, mouse_y, 32, true, 12, 0.1)
	end
end

function love.mousereleased(mouse_x, mouse_y, key)
	if key == "l" then
		local velocity = distance(line.pos[1][1], line.pos[1][2], line.pos[2][1], line.pos[2][2])
		local angle = angle(line.pos[2][1], line.pos[2][2], line.pos[1][1], line.pos[1][2])

		planet:new(line.pos[1][1], line.pos[1][2], 6, false, 3, 0.0, nil, nil, math.cos(angle) * velocity, math.sin(angle) * velocity)
	elseif key == "r" then

	end
end

function distance(x1, y1, x2, y2)
	return math.sqrt(((x2 - x1) * (x2 - x1)) + ((y2 - y1) * (y2 - y1)))
end

function angle(x1, y1, x2, y2)
	return math.atan2(y2-y1, x2-x1)
end

function random_bool()
	local i = math.random()
	if i >= 0.5 then return true else return false end
end

function average(t)
	local r = 0
	for i,v in ipairs(t) do
		r = r + v
	end
	return r / #t
end

function in_range(number, low, high)
	if number >= low and number <= high then 
		return true 
	else return false end
end

function longest_line(str)
	local lines = {}
	for i in str:gmatch("[^\r\n]+") do lines[#lines + 1] = i end
	table.sort(lines, long)
	return lines[1]
end

function lines(str)
	local l = 0
	for i in str:gmatch("[^\r\n]+") do l = l + 1 end
	return l
end

function long(str1, str2)
    return #str1 > #str2
end

function setAlpha(a)
	r, g, b, _a = love.graphics.getColor()
	love.graphics.setColor(r, g, b, a)
end