planet = {
	planets = {},
	physics = {
		world = false,
		objects = {}
	}
}

local remove_threshold = 512

function planet:new(x, y, radius, static, mass, restitution, color, name, xVel, yVel)
	static = static or false
	mass = mass or radius
	color = color or {love.math.random(90, 180), love.math.random(90, 180), love.math.random(90, 180), 255}
	name = name or "untitled"
	restitution = restitution or 0
	local p = {
		name = name,
		position = {x = x, y = y},
		radius = radius,
		mass = mass,
		static = static,
		color = color,
		gravity_radius = radius * (mass / 2),
		info_box = {
			x = x,
			y = y,
			visible = false,
			alpha = 0,
			fade_speed = 12,
			text = "",
			color = color,
		},
		show_trail = false,
		trail = {},
		trail_length = 64,
		trail_updateRate = 60,
		trail_tick = 0
	}
	planet:newPlanetObject(x, y, radius, static, xVel, yVel, restitution)
	self.planets[#self.planets + 1] = p
end

function planet:update(dt)
	if simulation then self.physics.world:update(dt) end
	for i,v in ipairs(self.planets) do
		--==] PHYSICS [==--
		if simulation then
			local force_x = {}
			local force_y = {}
			local ob = false
			for o,b in ipairs(self.planets) do
				if o ~= i then
					local dist = distance(v.position.x, v.position.y, b.position.x, b.position.y)
					local angle = angle(v.position.x, v.position.y, b.position.x, b.position.y)
					--if dist < (v.radius + b.gravity_radius) then
						force_x[#force_x + 1] = (v.position.x - b.position.x) * ((v.mass * b.mass) * 0.0005)
						force_y[#force_y + 1] = (v.position.y - b.position.y) * ((v.mass * b.mass) * 0.0005)
						ob = true
					--end
				end
			end

			if ob then self:applyForce(i, -average(force_x), -average(force_y)) end

			v.position.x = self.physics.objects[i].body:getX()
			v.position.y = self.physics.objects[i].body:getY()

			--Trail
			if v.show_trail then
				v.trail_tick = v.trail_tick + dt
				if v.trail_tick > (1 / v.trail_updateRate) then
					local xv, yv = self.physics.objects[i].body:getLinearVelocity()
					if math.floor(xv) ~= 0 and math.floor(yv) ~= 0 then
						v.trail[#v.trail + 1] = {x = v.position.x, y = v.position.y}
					end
					v.trail_tick = 0
				end
				if #v.trail > v.trail_length then
					table.remove(v.trail, 1)
				end
			end
		end

		--==] InfoBox [==--
		v.info_box.visible = false
		if distance(v.position.x, v.position.y, love.mouse.getX(), love.mouse.getY()) < (v.radius * 2) then
			v.info_box.visible = true
		end
		if v.info_box.visible then
			v.info_box.alpha = v.info_box.alpha + v.info_box.fade_speed * dt
			if v.info_box.alpha > 1 then v.info_box.alpha = 1 end
		else	
			v.info_box.alpha = v.info_box.alpha - v.info_box.fade_speed * dt
			if v.info_box.alpha < 0 then v.info_box.alpha = 0 end
		end
		--Info
		local vel_x, vel_y = self.physics.objects[i].body:getLinearVelocity()
		v.info_box.text = 	"Position: ["..math.floor(v.position.x).."]["..math.floor(v.position.y).."]\n"..
							"Velocity: ["..math.floor(vel_x).."]["..math.floor(vel_y).."]\n"..
							"Radius: "..v.radius.." Mass: "..v.mass
		--Off screen destroying
		if (v.position.x + v.gravity_radius) < -remove_threshold or (v.position.x - v.gravity_radius) > (screen.width + remove_threshold) or (v.position.y + v.gravity_radius) < -remove_threshold or (v.position.y - v.gravity_radius) > (screen.height + remove_threshold) then
			planet:destroy(i)
		end
	end
end

function planet:draw()
	for i,v in ipairs(self.planets) do
		--Trail
		if v.show_trail then
			for o,b in ipairs(v.trail) do
				if o > 1 then
					local a = (1 / v.trail_length) * o
					local l = ((v.radius / 2) / v.trail_length) * o
					love.graphics.setColor(v.color[1] * a, v.color[2] * a, v.color[3] * a, 255)
					love.graphics.setLineWidth(l)
					love.graphics.line(b.x, b.y, v.trail[o-1].x, v.trail[o-1].y)
				end
			end
		end

		love.graphics.setColor(v.color)
		love.graphics.circle("fill", v.position.x, v.position.y, v.radius)


		--if v.static then
			love.graphics.setColor(v.color[1], v.color[2], v.color[3])
			local l = love.graphics.getLineWidth()
			love.graphics.setLineWidth(v.radius / 4)
			love.graphics.setBlendMode("additive")
			setAlpha(128)
			love.graphics.circle("line", v.position.x, v.position.y, v.radius - (love.graphics.getLineWidth() / 2))
			love.graphics.setBlendMode("alpha")
			love.graphics.setLineWidth(l)
		--end
	end
	self:draw_infoBox()
end

function planet:draw_infoBox()
	for i,v in ipairs(self.planets) do
		if v.info_box.alpha > 0 then
			local f = love.graphics.getFont()
			local mouse_x, mouse_y = love.mouse.getPosition()
			local width = f:getWidth(longest_line(v.info_box.text))
			local height = f:getHeight() * (lines(v.info_box.text) + 1)
			mouse_y = mouse_y + 18
			mouse_x = mouse_x + 18
			--Info box
			love.graphics.setColor(v.color[1] / 2, v.color[2] / 2, v.color[3] / 2, v.color[4] * v.info_box.alpha)
			love.graphics.rectangle("fill", mouse_x, mouse_y, width, height)

			love.graphics.setColor(255, 255, 255, 255 * v.info_box.alpha)
			love.graphics.print(v.info_box.text, mouse_x, mouse_y + f:getHeight())

			--Name box
			love.graphics.setColor(0, 0, 0, 64 * v.info_box.alpha)
			love.graphics.rectangle("fill", mouse_x, mouse_y, width, f:getHeight())
			love.graphics.setColor(v.color[1], v.color[2], v.color[3], v.color[4] * v.info_box.alpha)
			local n = "["..i.."] "..v.name
			if v.static then n = n.." (static)" end
			love.graphics.print(n, mouse_x, mouse_y)

			love.graphics.setColor(v.color[1], v.color[2], v.color[3], 255 * v.info_box.alpha)
			love.graphics.setLineStyle("rough")
			love.graphics.setLineWidth(1)
			love.graphics.rectangle("line", mouse_x, mouse_y, width, height)
		end
	end
end

--love.physics things

function planet:newWorld(gx, gy, sleep, meter)
	love.physics.setMeter(meter)
	self.physics.world = love.physics.newWorld(gx, gy, sleeep)
end

function planet:newPlanetObject(x, y, radius, static, xVel, yVel, restitution)
	local type = "dynamic"
	if static then type = "static" end
	xVel = xVel or 0
	yVel = yVel or 0
	local p = {
		body = love.physics.newBody(self.physics.world, x, y, type),
		shape = love.physics.newCircleShape(radius)
	}
	p.fixture = love.physics.newFixture(p.body, p.shape, 1)
	p.body:setLinearVelocity(xVel, yVel)
	p.fixture:setRestitution(restitution)
	self.physics.objects[#self.physics.objects + 1] = p
end

function planet:draw_physics_debug()
	for i,v in ipairs(self.physics.objects) do
		--local xv, yv = v.body:getLinearVelocity()

		love.graphics.setColor(self.planets[i].color[1], self.planets[i].color[2], self.planets[i].color[3], 12)
		love.graphics.setLineStyle("smooth")
		love.graphics.setLineWidth(4)
		love.graphics.setBlendMode("additive")
		love.graphics.circle("fill", v.body:getX(), v.body:getY(), self.planets[i].gravity_radius)
		setAlpha(24)
		love.graphics.circle("line", v.body:getX(), v.body:getY(), self.planets[i].gravity_radius - 2)
		love.graphics.setBlendMode("alpha")
	end
end

function planet:applyForce(i, x, y)
	if in_range(i, 1, #self.physics.objects) then
		self.physics.objects[i].body:applyForce(x, y)
	end
end

function planet:destroy(i)
	if in_range(i, 1, #self.physics.objects) then
		table.remove(self.planets, i)
		table.remove(self.physics.objects, i)
		console:print("planet".."["..i.."] destroyed")
	end
end

function planet:setName(i, name)
	if in_range(i, 1, #self.physics.objects) then
		self.planets[i].name = name
	end
end

function planet:setMass(i, mass)
	if in_range(i, 1, #self.physics.objects) then
		self.planets[i].mass = mass
		self.planets[i].gravity_radius = self.planets[i].radius * (self.planets[i].mass / 4)
	end
end

function planet:setRadius(i, radius)
	if in_range(i, 1, #self.physics.objects) then
		self.planets[i].radius = radius
		self.planets[i].gravity_radius = self.planets[i].radius * (self.planets[i].mass / 4)
	end
end

function planet:setRestitution(i, r)
	if in_range(i, 1, #self.physics.objects) then
		self.physics.objects[i].fixture:setRestitution(r)
	end
end

function planet:countStaticPlanets()
	local p = 0
	for i,v in ipairs(self.planets) do
		if v.static then p = p + 1 end
	end
	return p
end

function planet:countDynamicPlanets()
	local p = 0
	for i,v in ipairs(self.planets) do
		if not v.static then p = p + 1 end
	end
	return p
end

function planet:countPlanets()
	return #self.planets or 0
end

function planet:clearWorld()
	self.planets = {}
	self.physics.objects = {}
end

function planet:addMoon(i)
	local p = self.planets[i]
	local x, y = p.position.x, p.position.y - p.gravity_radius * 0.8
	local radius = 6
	local mass = 3
	local xVel = (screen.height - y) / (p.mass * mass) * (p.radius / math.pi)
	planet:new(x, y, radius, false, mass, 0, nil, nil, xVel)
	--planet:new(line.pos[1][1], line.pos[1][2], 6, false, 3, 0, nil, nil, math.cos(angle) * velocity, math.sin(angle) * velocity) 0.002
end